@@ -8,6 +8,7 @@ Author: Florent Xicluna
 from __future__ import with_statement
 
 import _ast
+import atexit
 import collections
 import functools
 import optparse
@@ -31,7 +32,7 @@ except ImportError:     # Python 2
     int_types = int, long
 
 
-__version__ = '1.5.1'
+__version__ = '1.5.2'
 __all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service',
            'format_exception', 'read_config', 'start_openerp_services']
 
@@ -210,7 +211,7 @@ def read_config(section=None):
     return (server, env['database'], env['username'], env.get('password'))
 
 
-def start_openerp_services(options=None):
+def start_openerp_services(options=None, appname=None):
     """Initialize the OpenERP services.
 
     Import the ``openerp`` package and load the OpenERP services.
@@ -221,6 +222,8 @@ def start_openerp_services(options=None):
     import openerp
     if not openerp.osv.osv.service:
         os.environ['TZ'] = 'UTC'
+        if appname is not None:
+            os.environ['PGAPPNAME'] = appname
         options = options.split() if options else []
         openerp.tools.config.parse_config(options)
         if openerp.release.version_info < (7,):
@@ -229,6 +232,12 @@ def start_openerp_services(options=None):
             openerp.service.web_services.start_web_services()
         else:
             openerp.service.start_internal()
+
+        def close_all():
+            for db in openerp.modules.registry.RegistryManager.registries:
+                openerp.sql_db.close_db(db)
+        atexit.register(close_all)
+
     return openerp
 
 
@@ -370,11 +379,6 @@ class Client(object):
         if isinstance(server, basestring) and server[-1:] == '/':
             server = server.rstrip('/')
         self._server = server
-        self._db = ()
-        self._environment = None
-        self.user = None
-        self._execute = None
-        self._models = {}
         major_version = None
 
         def get_proxy(name):
@@ -392,6 +396,7 @@ class Client(object):
         self._object = get_proxy('object')
         self._report = get_proxy('report')
         self._wizard = get_proxy('wizard') if major_version < '7.0' else None
+        self.reset()
         if db:
             # Try to login
             self._login(user, password=password, database=db)
@@ -406,11 +411,17 @@ class Client(object):
         """
         (server, db, user, password) = read_config(environment)
         if server[0] == 'local':
-            server = start_openerp_services(server[1])
+            appname = os.path.basename(__file__)
+            server = start_openerp_services(server[1], appname=appname)
         client = cls(server, db, user, password, verbose=verbose)
         client._environment = environment
         return client
 
+    def reset(self):
+        self.user = self._environment = None
+        self._db, self._models = (), {}
+        self._execute = self._exec_workflow = None
+
     def __repr__(self):
         return "<Client '%s#%s'>" % (self._server or '', self._db)
 
@@ -439,8 +450,8 @@ class Client(object):
             print('Error: Invalid username or password')
             return
         if self._db != database:
+            self.reset()
             self._db = database
-            self._environment = None
         self.user = user
 
         # Authenticated endpoints
@@ -531,8 +542,7 @@ class Client(object):
             """Connect to another environment and replace the globals()."""
             if env:
                 # Safety measure: turn down the previous connection
-                previous = global_vars['client']
-                previous._execute = previous._exec_workflow = None
+                global_vars['client'].reset()
                 client = self.from_config(env, verbose=self.db._verbose)
             else:
                 client = self
@@ -593,6 +603,7 @@ class Client(object):
         Method `params` are allowed.  If needed, keyword
         arguments are collected in `kwargs`.
         """
+        assert self.user, 'Not connected'
         assert isinstance(obj, basestring)
         assert isinstance(method, basestring) and method != 'browse'
         context = kwargs.pop('context', None)
@@ -653,6 +664,7 @@ class Client(object):
         Argument `obj` is the name of the model.  The `signal`
         is sent to the object identified by its integer ``id`` `obj_id`.
         """
+        assert self.user, 'Not connected'
         assert isinstance(obj, basestring) and isinstance(signal, basestring)
         return self._exec_workflow(obj, signal, obj_id)
 
@@ -894,6 +906,12 @@ class Client(object):
         wrapper.__doc__ %= method
         return wrapper.__get__(self, type(self))
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, tb):
+        self.reset()
+
 
 class Model(object):
     """The class for OpenERP models."""
@@ -1412,7 +1430,7 @@ class Record(object):
 
     def __eq__(self, other):
         return (isinstance(other, Record) and
-                self.id == other.id and self._model_name == other._model_name)
+                self.id == other.id and self._model is other._model)
 
 
 def _interact(global_vars, use_pprint=True, usage=USAGE):
@@ -1449,7 +1467,6 @@ def _interact(global_vars, use_pprint=True, usage=USAGE):
     except Exception:
         pass
     else:
-        import atexit
         if rl.get_history_length() < 0:
             rl.set_history_length(int(os.environ.get('HISTSIZE', 500)))
         # better append instead of replace?
